@@ -31,6 +31,14 @@ const PlaceStore = imports.placeStore;
 const SearchPopup = imports.searchPopup;
 const Utils = imports.utils;
 
+Geocode.Location.prototype.equals = function(location) {
+    return (location.latitude === this.latitude &&
+            location.longitude === this.longitude);
+};
+
+// Matches coordinates string with the format "<lat>, <long>"
+const COORDINATES_REGEX = /^\s*(\-?\d+(?:\.\d+)?)\s*,\s*(\-?\d+(?:\.\d+)?)\s*$/;
+
 const PlaceEntry = new Lang.Class({
     Name: 'PlaceEntry',
     Extends: Gtk.SearchEntry,
@@ -38,25 +46,31 @@ const PlaceEntry = new Lang.Class({
         'place': GObject.ParamSpec.object('place',
                                           'Place',
                                           'The selected place',
-                                          GObject.ParamFlags.READWRITE,
+                                          GObject.ParamFlags.READABLE |
+                                          GObject.ParamFlags.WRITABLE,
                                           Geocode.Place)
     },
 
     set place(p) {
+        if (!this._place && !p)
+            return;
+
+        if (this._place && p && this._place.location.equals(p.location))
+            return;
+
         if (p) {
-            if (!this.place ||
-                (this.place.location.latitude != p.location.latitude ||
-                 this.place.location.longitude != p.location.longitude)) {
-                this._place = p;
-                this.text   = p.name;
-                this.notify("place");
-            }
-        } else {
-            this._place = p;
-            this.text   = "";
-            this.notify("place");
-        }
+            if (p.name) {
+                this.text = p.name;
+                Application.placeStore.addRecent(p);
+            } else
+                this.text = p.location.latitude + ', ' + p.location.longitude;
+        } else
+            this.text = '';
+
+        this._place = p;
+        this.notify('place');
     },
+
     get place() {
         return this._place;
     },
@@ -75,6 +89,9 @@ const PlaceEntry = new Lang.Class({
             props.primary_icon_name = null;
         delete props.loupe;
 
+        let parseOnFocusOut = props.parseOnFocusOut;
+        delete props.parseOnFocusOut;
+
         props.completion = this._createCompletion();
         this.parent(props);
 
@@ -87,6 +104,13 @@ const PlaceEntry = new Lang.Class({
             if (this.text.length === 0)
                 this.place = null;
         }).bind(this));
+
+        if (parseOnFocusOut) {
+            this.connect('focus-out-event', (function() {
+                this._parse();
+                return false;
+            }).bind(this));
+        }
     },
 
     _createCompletion: function() {
@@ -123,12 +147,45 @@ const PlaceEntry = new Lang.Class({
         return popover;
     },
 
-    _onActivate: function() {
+    _validateCoordinates: function(lat, lon) {
+        return lat <= 90 && lat >= -90 && lon <= 180 && lon >= -180;
+    },
+
+    _parseCoordinates: function(text) {
+        let match = text.match(COORDINATES_REGEX);
+
+        if (match) {
+            let latitude = parseFloat(match[1]);
+            let longitude = parseFloat(match[2]);
+
+            if (this._validateCoordinates(latitude, longitude)) {
+                return new Geocode.Location({ latitude: latitude,
+                                              longitude: longitude });
+            } else
+                return null;
+        } else
+            return null;
+    },
+
+    _parse: function() {
         if (this.text.length === 0) {
             this.place = null;
-            return;
+            return true;
         }
 
+        let parsedLocation = this._parseCoordinates(this.text);
+        if (parsedLocation) {
+            this.place = new Geocode.Place({ location: parsedLocation });
+            return true;
+        }
+
+        return false;
+    },
+
+    _onActivate: function() {
+        if (this._parse())
+            return;
+
         let bbox = this._mapView.view.get_bounding_box();
 
         this._popover.showSpinner();
