@@ -76,19 +76,6 @@ const TurnPointMarker = new Lang.Class({
     }
 });
 
-const InstructionMarker = new Lang.Class({
-    Name: 'InstructionMarker',
-    Extends: TurnPointMarker,
-
-    _init: function(params) {
-        this.parent(params);
-
-        this.bubble.connect_after('closed', (function() {
-            this.destroy();
-        }).bind(this));
-    }
-});
-
 const DestinationMarker = new Lang.Class({
     Name: 'DestinationMarker',
     Extends: TurnPointMarker,
@@ -105,17 +92,14 @@ const DestinationMarker = new Lang.Class({
             this._onMarkerDrag();
         }).bind(this));
 
-        let iconName = Path.ICONS_DIR + '/' + this.iconName + '.svg';
-        this.add_actor(Utils.CreateActorFromImageFile(iconName));
+        this.add_actor(Utils.CreateActorFromIconName(this.iconName));
     },
 
     _onMarkerDrag: function() {
         let query = Application.routeService.query;
-        let name = '[' + this.latitude.toFixed(3) + ', ' + this.longitude.toFixed(3) + ']';
         let place = new Geocode.Place({
-                        name: name,
-                        location: new Geocode.Location({ latitude: this.latitude,
-                                                         longitude: this.longitude }) });
+                        location: new Geocode.Location({ latitude: this.latitude.toFixed(5),
+                                                         longitude: this.longitude.toFixed(5) }) });
 
         this._queryPoint.place = place;
     }
